% This function creates a synthetic data vector according to input parameters
function [x_t,t,tk,xk,times,x_t_without_noise] = CreateSyntheticData(K,tau,B,fs,type,calibrate_flag,sigma,bandpass_flag,fc,sigma_analog_noise)
%% Create times of pulses
if calibrate_flag
    K=1;
    temp = linspace(0,tau,K+2);
    tk_temp = temp(2:(end-1)).';
    xk = ones(K,1);
elseif strcmp(type,'uniformly_distributed')
    temp = linspace(0,tau,K+2);
    tk_temp = temp(2:(end-1)).';
    xk = ones(K,1);
elseif strcmp(type,'random')
    interval_margin = 2/B;
    tk_temp = sort(interval_margin + (tau-2*interval_margin)*rand(K,1));
%     xk = 0.5 + 0.5*abs(randn(K,1));
%     xk = randsrc(K,1).*(0.5 + 0.5*abs(randn(K,1))); % Negative amplitudes too
% % If you don't have randsrc in your matlab use this
xk = (2*(random('unid',2,K,1) == 1) - 1).*(0.5 + 0.5*abs(randn(K,1))); % Negative amplitudes too
elseif strncmp(type,'load_',length('load_'))
    % Read (tk,xk)from file
      load(type((length('load_')+1):end));
      tk_temp = tk;
%       save('5G_random_for_paper_periodic_141009','tk','xk');
end
%% Samples that mimic ultrasound imaging
Nsamples = ceil(tau*fs);
% period = Nsamples/fs;
t = (0:Nsamples-1)/fs;
tk = round(tk_temp*fs)/fs;
%tk=tk_temp;
times = repmat(t.',1,K) - repmat(tk.',length(t),1);
if bandpass_flag
    x_t_without_noise = exp( - ( (times).^2 / (2*sigma^2) ) ).*cos(2*pi*fc*times)*xk;
else
    x_t_without_noise = exp( - ( (times).^2 / (2*sigma^2) ) )*xk;
end
if calibrate_flag
  sigma_analog_noise = 0;
end
x_t = x_t_without_noise + sigma_analog_noise*randn(size(x_t_without_noise));
% x_periodic = repmat(x_t,num_of_periods,1);
figure('Name',sprintf('Original signal x_t - sum of %d guassians',K)); 
plot(t,x_t,'LineWidth',1.5); 
xlabel('time [units of \tau]'); ylabel('Amplitude');