%% Introduction
% This function applies SoS method by replicating the signal itself,
% obtaining 3 periods of the signal, which is enough for narrow gaussians.
%% The function itself
function [tk_FRI,tk_FRI_Cadzow,xk_est]=SoS_Idea_Periodic_Func(x_t,K,tau,B,fs,Nsamples,t,times,bandpass_flag,fc,N,T,sigma,tk,xk,tk_known_flag,sigma_analog_noise,digital_SNR,b_k,plot_flag)
num_of_periods = 3; % number of signal periods
num_of_filter_periods = 1;
M=floor(B*tau/2);
x_t = x_t(:).';
x_3_periods = repmat(x_t,1,num_of_periods); %create 3 periods of signal
% x_3_periods = [zeros(size(x_t));x_t;zeros(size(x_t))]; %create 3 periods of signal
t_3_periods = ((0:num_of_periods*Nsamples-1)-Nsamples)/fs;
%t_2_periods = (0:2*Nsamples-1)/fs;

if plot_flag
    figure('Name','Original signal x_t (for ultrasound - after demodulation');
    plot(t,x_t,'LineWidth',1.5);
    xlabel('time [units of \tau]'); ylabel('Amplitude');
end

if tk_known_flag
    figure('Name',sprintf('3 periods of x_t - sum of %d guassians',K));
else
    figure('Name','3 periods of original ultrasonic signal x_t after demodulation');
end
plot(t_3_periods,real(x_3_periods),'LineWidth',1.5); 
xlabel('time [units of \tau]'); ylabel('Amplitude');
ax = axis;
%% Apply SoS filter on x_3_periods
ch = 1;
K_set = -M:M;
% [TIME,K_SET] = meshgrid(t-tau/2,K_set);


% exp_jkt = exp(1j*K_SET.*TIME).';
%my_filter = (1./exp_jkt)*b_k;

t_max_filter = floor(tau*fs/2);
t_filter = (-t_max_filter:t_max_filter)/fs;
[FILTER_TIME,FILTER_K] = meshgrid(t_filter,K_set);

%% Filter fourier series coeff.
% load 'F_k_double_18_5' %F_k_double_15_2 is a gaussian filter with sigma = 0.05
% b_k = F_k_double(:);
% gauss_filter = exp(-t_filter.^2/(2*gauss_sigma^2));
% my_mat = conj(Create_V_tau_matrix(t_filter,K_set,tau)).';
% gauss_filter_approx = my_mat*F_k_double.';
% figure('Name','The filter and its finite fourier series coeff. approximation'); 
% plot(t_filter,gauss_filter)
% hold all
% plot(t_filter,gauss_filter_approx)
%% Applying filter - cont.
% my_filter = exp(1j*(-M)*2*pi/tau*(t_filter)).*(1 -
% exp(1j*length(K_set)*2*pi/tau*(t_filter)))./(1 - exp(1j*2*pi/tau*(t_filter)));
% my_filter(t_filter==0) = B;
% % The last two rows were in the previous version (2_5)

% b_k = ones(length(K_set),1);
% b_k = 1.1+cos(2*pi*K_set/(tau*length(K_set)*2)).';
figure('Name','Stem of b_k - filter coeff.')
stem(b_k)
% my_filter = exp(1j*2*pi*FILTER_K.'.*FILTER_TIME.'/tau)*conj(b_k);
my_filter = SumOfSincsFilter(t_filter,tau,K_set,b_k,plot_flag,num_of_filter_periods);
figure('Name','Real part of filter'); 
plot(t_filter,real(my_filter))
xlabel('Timeline [units of \tau]')
ylabel('Amplitude [A.U.]')


[y_t_preliminary,zf] = filter(my_filter,1,x_3_periods); 
% y_t_preliminary = y_t_preliminary/
delay=floor(length(my_filter)/2); 

y_t_preliminary = y_t_preliminary(:);
y_t = y_t_preliminary( (Nsamples+1:2*Nsamples) + delay);


if plot_flag
    figure('Name','Output vs. 3 period input');
    plot(x_3_periods)
    hold all
    plot(real(y_t_preliminary(delay+1:end,ch))) 
    %plot(imag(y_t_preliminary(delay:end,ch)))
    % plot(real(my_filter))
    % plot(imag(my_filter))
    figure('Name','Filter output y_t vs. input signal x_t')
    plot(x_t)
    hold all
    plot(real(y_t))
end
%% Low frequency samples
if (digital_SNR<1e15)
    y_t_for_sampling = y_t + sqrt(mean(abs(y_t).^2)/digital_SNR)*randn(size(y_t));
else
    y_t_for_sampling = y_t;
end

t_low_freq = (0:N-1)*T;
%x_n = x_2_periods(fs*T*((0:N-1) + floor(N/2))+1);
c_n = y_t_for_sampling(fs/B*(0:N-1) + 1);
%y_n_sinc = z_t_sinc(fs*T*(0:N-1)+1).';
%y_n = z_t(t_low_freq);
% C_K = fftshift(fft(c_n))/length(c_n);
C_K = fftshift(fft(c_n));
C_K = C_K(:);
% H = diag(ones(length(K_set),1)); % This should be a matrix with H(2\pi k/T) on its diagonal
H = 1/T*diag(sigma*exp(-(2*pi*K_set/tau).^2*sigma^2/2)); % The fourier transform of exp(-t^2/(2*sigma^2))
% is: sigma*exp(-w^2*sigma^2/2). On the diagonal of H is this function
% evaluated at 2*pi*k, for each k in K_set
S = sqrt(2*pi)/T*diag(b_k); % Filter coefficients
%T/sqrt(2*pi)*
% C_K_calibrated = H^(-1)*S^(-1)*C_K *sigma;
C_K_calibrated = H^(-1)*S^(-1)*C_K;
%% figure
if plot_flag
    figure('Name','3 periods input and output');
    plot(t_3_periods,x_3_periods)
    hold all
    plot(t_3_periods(1:end-delay),real(y_t_preliminary(delay+1:end)))
end

figure('Name','Filter output and low rate samples'); 
% plot(t,x_t)
% hold all
plot(t,real(y_t)/max(abs(real(y_t))),'LineWidth',1.5)
hold all
stem(t_low_freq,real(c_n)/max(abs(real(y_t))),'o','LineWidth',1,'MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',8)
xlabel('time [units of \tau]')
ylabel('Amplitude')
ax2 = axis;
ax2(3:4) = ax(3:4);
axis(ax2)
legend('Filter output','Low rate samples');
%% Estimate tk's
%a = FRI_classic(C_K,K,tau);
tk_FRI = FRI_classic(C_K_calibrated,K,tau);
tk_FRI_Cadzow = FRI_Cadzow(C_K_calibrated,B,T,K,tau);

if plot_flag
    disp('----------------------------------------------------------------------------------------------------------');
    disp('----------------------------------------------------------------------------------------------------------');
    disp(sprintf('Annihilating Filter - Sqrt of Squared error = %0.3g', norm(tk_FRI - tk)))
    %disp(sprintf('Sqrt of Squared error Using chosen samples= %0.3g', norm(tk_FRI_chosen - tk)))
    disp('------------------------------------------');
    % tk_FRI_Cadzow_chosen = FRI_Cadzow(ym_chosen,B,T,K,tau);
    disp(sprintf('Cadzow - Sqrt of Squared error = %0.3g', norm(tk_FRI_Cadzow - tk)))
    % disp(sprintf('Cadzow - Sqrt of Squared error Using chosen samples= %0.3g', norm(tk_FRI_Cadzow_chosen - tk)))
    disp('------------------------------------------');
end

% tk_mp=sort(mod(-MP_fb(C_K,round(1/3*N),K)/(2*pi)*tau,1));
% disp(sprintf('Matrix Pencil - Sqrt of Squared error Using all samples = %0.3g', norm(tk_mp - tk)))
% % tk_mp_chosen=sort(mod(-MP_fb(ym_chosen,round(1/3*N),K)/(2*pi)*tau,1));
% % disp(sprintf('Matrix Pencil - Sqrt of Squared error Using chosen samples = %0.3g', norm(tk_mp_chosen - tk)))
% disp('------------------------------------------');

% tk_kt=sort(mod(-KT_fb(C_K.',round(1/3*N),K)/(2*pi)*tau,1));
% disp(sprintf('Kumaresan-Tufts - Sqrt of Squared error Using all samples = %0.3g', norm(tk_kt - tk)))
% % tk_kt_chosen=sort(mod(-KT_fb(ym_chosen.',round(1/3*N),K)/(2*pi)*tau,1));
% % disp(sprintf('Kumaresan-Tufts - Sqrt of Squared error Using chosen samples = %0.3g', norm(tk_kt_chosen - tk)))
% disp('------------------------------------------');

%% Estimate amplitudes xk
tk_est = tk_FRI_Cadzow;
% ym = ym_chosen;
% m_vec = 0:(length(C_K)-1); % DFT scale
m_vec = K_set;
[L,M_Mat]=meshgrid(tk_est,m_vec);
% [L,M_Mat]=meshgrid(tk_est,K_set);
my_mat = exp(-1i*2*pi/tau*L.*M_Mat);
% xk_est = abs(pinv(my_mat)*C_K_calibrated)*T;
xk_est = pinv(my_mat)*C_K_calibrated/length(x_t)/T^2*2;


% no need to multiply by tau since C_k is actually tau*(fourier series
% coeff.). 

%% Plot estimated signal vs. original
%tk_estimated = round(tk_FRI*fs)/fs;
% tk_est=tk_FRI;
times_est = repmat(t.',1,K) - repmat(tk_est.',length(t),1);
x_t_estimated = exp( - ( (times_est).^2 / (2*sigma^2) ) )*real(xk_est);
% x_t_estimated = exp( - ( (times_est).^2 / (2*sigma^2) ) ).*cos(2*pi*fc*times)*real(xk_est);

if tk_known_flag
    figure('Name','Original signal x_t vs. estimated signal');
    plot(t,x_t,'LineWidth',2.5);
    hold all
    %pause
    plot(t,x_t_estimated,'--r','LineWidth',2,'MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',1)
    % plot(t,x_t_without_noise,':c','LineWidth',3,'MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',1)
    legend('Original Signal','Reconstructed Signal');
    xlabel('time [units of \tau]'); ylabel('Amplitude');

%     figure('Name','Original signal x_t + estimated positions of gaussians');
%     plot(t,x_t,'LineWidth',1.5)
%     hold all
%     plot(tk_est,real(xk_est),'o','LineWidth',1.5,'MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',8)
%     legend('Original Signal','Estimated times and amplitudes')%,'Location','SouthOutside');
%     xlabel('time [units of \tau]'); ylabel('Amplitude');

    figure('Name','Original signal x_t + estimated positions of gaussians');
    plot(t,x_t,'LineWidth',1.5)
    hold all
    stem(tk_est,real(xk_est),'o','LineWidth',0.1,'MarkerEdgeColor','k','MarkerFaceColor','g','MarkerSize',8)
    % legend('Original Signal','Estimated times and amplitudes')%,'Location','SouthOutside');
    legend('Original Signal','Estimated parameters')%,'Location','SouthOutside');
    xlabel('time [units of \tau]'); ylabel('Amplitude');
end

if plot_flag
    disp(sprintf('Error in apmlitudes, xk, Using estimated tk''s = %0.3g', norm(xk_est - xk)))
    disp('------------------------------------------');
end

%% Analytically
% [TK,K_SET] = meshgrid(tk,K_set);
% V_tau = exp(-1j*2*pi*K_SET.*TK/tau);
if tk_known_flag
    V_tau = Create_V_tau_matrix(tk,K_set,tau);
    X_k = H*V_tau*xk;
    X_k = X_k(:);
    noisy_X_k = X_k + sigma_analog_noise*randn(size(X_k));
    tk_analytic_X_k = FRI_classic(H^-1*noisy_X_k,K,tau);
    xk_est_analytic_X_k = pinv(Create_V_tau_matrix(tk_analytic_X_k,K_set,tau))*noisy_X_k;
    
    % [TAU_s,K_SET_s] = meshgrid((0:length(K_set)-1)*tau/N,K_set);
    % V_tau_s = exp(1j*2*pi*K_SET_s.*TAU_s/tau).';
    V_tau_s = conj(Create_V_tau_matrix((0:length(K_set)-1)*tau/N,K_set,tau)).';
    % V_tau_s3 = Create_V_tau_matrix((0:length(K_set)-1)*tau/N,-K_set,tau);
    % S = diag(b_k);
    c_n_analytic = V_tau_s*S*X_k + sigma_analog_noise*randn(size(X_k));
    C_K_analytic = fftshift(fft(c_n_analytic));
    C_K_analytic = C_K_analytic(:);
    C_K_analytic_calibrated = H^(-1)*S^(-1)*C_K_analytic;
    tk_analytic_c_n = FRI_classic(C_K_analytic_calibrated,K,tau);
    %xk_est_analytic_c_n = pinv(Create_V_tau_matrix(tk_analytic_c_n,K_set,tau))*C_K_analytic_calibrated;
    xk_est_analytic_c_n = pinv(Create_V_tau_matrix(tk_analytic_c_n,K_set,tau))*C_K_analytic_calibrated*T;
    
    if plot_flag
        disp('----------------------------------------------------------------------------------------------------------');
        disp(sprintf('Ann. Filter using analytic samples X_k in frequency. Times error = %0.3g', norm(tk_analytic_X_k - tk)))
        disp(sprintf('Amplitudes error = %0.3g', norm(xk_est_analytic_X_k - xk)))
        disp('------------------------------------------');
        disp(sprintf('Ann. Filter using analytic samples c_n (ifft of X_k) + noise. Times error = %0.3g', norm(tk_analytic_c_n - tk)))
        disp(sprintf('Amplitudes error = %0.3g', norm(xk_est_analytic_c_n - xk)))
        %analytic_X_k = fft(ifft(f
    end
end